@@ -16,31 +16,22 @@ import ssl
 import subprocess
 import sys
 import tempfile
-
-from typing import (
-    Any,
-    Callable,
-    Coroutine,
-    Dict,
-    Iterator,
-    List,
-    NamedTuple,
-    Optional,
-    Union,
-    Type,
-    TypeVar,
-)
+import typing
 
 
-PathStr = Union[str, "os.PathLike[Any]"]
-PathList = List[PathStr]
-_TYPING_USED = (Any,)
+if typing.TYPE_CHECKING:
+    from collections.abc import Callable, Coroutine, Iterator
+    from typing import Any, Final, TypeVar, Union
 
-VERSION = "0.1.1"
+    PathStr = Union[str, os.PathLike[Any]]
+    PathList = list[PathStr]
 
-DEFAULT_PROG = pathlib.Path("/usr/bin/stunnel4")
 
-RE_VERSION = re.compile(
+VERSION: Final = "0.1.3"
+
+DEFAULT_PROG: Final = pathlib.Path("/usr/bin/stunnel4")
+
+RE_VERSION: Final = re.compile(
     r""" ^
     stunnel \s+
     (?P<version> (?: [5-9] | [1-9][0-9]* ) \. \S+ )
@@ -49,7 +40,7 @@ RE_VERSION = re.compile(
     re.X,
 )
 
-RE_LINE_IDX = re.compile(r" ^ Hello \s+ (?P<idx> 0 | [1-9][0-9]* ) $ ", re.X)
+RE_LINE_IDX: Final = re.compile(r" ^ Hello \s+ (?P<idx> 0 | [1-9][0-9]* ) $ ", re.X)
 
 
 @dataclasses.dataclass(frozen=True)
@@ -130,35 +121,37 @@ class TestConnection:
     encrypted: bool
     hostname: str
     port: int
-    peer: Optional[str]
-    msgq: asyncio.Queue[ListenerClientEvent]  # pylint: disable=unsubscriptable-object
+    peer: str | None
+    msgq: asyncio.Queue[ListenerClientEvent]
 
 
-class TestConnections(NamedTuple):
+@dataclasses.dataclass(frozen=True)
+class TestConnections:
     """The various states of the test connections."""
 
-    by_id: Dict[int, TestConnection]
-    by_peer: Dict[str, TestConnection]
-    pending: Dict[str, List[ListenerClientEvent]]
-    unresolved: Dict[int, TestConnection]
+    by_id: dict[int, TestConnection]
+    by_peer: dict[str, TestConnection]
+    pending: dict[str, list[ListenerClientEvent]]
+    unresolved: dict[int, TestConnection]
 
 
-class Config(NamedTuple):
+@dataclasses.dataclass(frozen=True)
+class Config:
     """Runtime configuration for the stunnel test."""
 
     certdir: pathlib.Path
-    children: Dict[int, asyncio.subprocess.Process]  # pylint: disable=no-member
-    mainq: asyncio.Queue[Event]  # pylint: disable=unsubscriptable-object
+    children: dict[int, asyncio.subprocess.Process]
+    mainq: asyncio.Queue[Event]
     program: pathlib.Path
-    tasks: Dict[str, asyncio.Task[None]]  # pylint: disable=unsubscriptable-object
+    tasks: dict[str, asyncio.Task[None]]
     tempd: pathlib.Path
-    utf8_env: Dict[str, str]
+    utf8_env: dict[str, str]
 
 
 @contextlib.contextmanager
 def parse_args() -> Iterator[Config]:
     """Parse the command-line arguments."""
-    parser = argparse.ArgumentParser()
+    parser: Final = argparse.ArgumentParser()
     parser.add_argument(
         "--certdir",
         type=pathlib.Path,
@@ -172,11 +165,11 @@ def parse_args() -> Iterator[Config]:
         help=f"the path to the stunnel executable to use (default: {DEFAULT_PROG}",
     )
 
-    args = parser.parse_args()
+    args: Final = parser.parse_args()
 
     # Generally we'd use the utf8_locale Python library here, but, well,
     # this is Debian, right?
-    utf8_env = dict(os.environ)
+    utf8_env: Final = dict(os.environ)
     utf8_env.update({"LC_ALL": "C.UTF-8", "LANGUAGE": ""})
 
     with tempfile.TemporaryDirectory(prefix="struntime.") as tempd_name:
@@ -195,34 +188,34 @@ def parse_args() -> Iterator[Config]:
 async def get_stunnel_version(cfg: Config) -> str:
     """Obtain the version of stunnel."""
     print(f"Trying to obtain the version of {cfg.program}")
-    cmd = [str(cfg.program), "-version"]
-    cmd_str = " ".join(shlex.quote(word) for word in cmd)
+    cmd: Final = [str(cfg.program), "-version"]
+    cmd_str: Final = shlex.join(cmd)
     try:
-        proc = await asyncio.create_subprocess_exec(
+        proc: Final = await asyncio.create_subprocess_exec(
             *cmd,
             stdin=subprocess.DEVNULL,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
             env=cfg.utf8_env,
         )
-    except (IOError, subprocess.CalledProcessError) as err:
+    except (OSError, subprocess.CalledProcessError) as err:
         sys.exit(f"Could not start `{cmd_str}`: {err}")
 
     print(f"Started `{cmd_str}` as process {proc.pid}")
     b_out, b_err = await proc.communicate()
-    assert b_out is not None and b_err is not None
-    p_out, p_err = b_out.decode("UTF-8"), b_err.decode("UTF-8")
+    assert b_out is not None and b_err is not None  # noqa: PT018,S101
+    p_out, p_err = b_out.decode(), b_err.decode()
 
-    rcode = await proc.wait()
+    rcode: Final = await proc.wait()
     if rcode != 0:
-        print(b_out.decode("UTF-8"))
-        print(b_err.decode("UTF-8"), file=sys.stderr)
+        print(b_out.decode())
+        print(b_err.decode(), file=sys.stderr)
         sys.exit(f"`{cmd_str}` exited with code {rcode}")
 
     if p_out:
         sys.exit(f"`{cmd_str}` produced output on its stdout stream:\n{p_out}")
 
-    lines = p_err.splitlines()
+    lines: Final = p_err.splitlines()
     if not lines:
         sys.exit(f"Expected at least one line of output from `{cmd_str}`")
     for line in lines:
@@ -250,7 +243,7 @@ async def find_listening_port(
                 port=port,
                 reuse_address=True,
             )
-        except IOError as err:
+        except OSError as err:
             print(f"[listener] - {port} failed: {err}")
 
     raise Exception("Could not find a port to listen on")
@@ -264,8 +257,8 @@ async def start_listener(cfg: Config) -> None:
         try:
             print("[listener] A client connected from somewhere")
             peer_addr, peer_port = writer.get_extra_info("peername")[:2]
-            peer = f"[{peer_addr}]:{peer_port}"
-        except Exception as err:  # pylint: disable=broad-except
+            peer: Final = f"[{peer_addr}]:{peer_port}"
+        except Exception as err:  # noqa: BLE001
             print(f"[listener] Complaining about the client: {err}")
             await cfg.mainq.put(
                 FatalEvent(
@@ -282,7 +275,7 @@ async def start_listener(cfg: Config) -> None:
 
             print(f"[{peer}] Let them tell us something...")
             line = await reader.readline()
-            match = RE_LINE_IDX.match(line.decode("UTF-8"))
+            match: Final = RE_LINE_IDX.match(line.decode())
             if not match:
                 sys.exit(f"Unexpected message from {peer!r}: {line!r}")
             print(f"[{peer}] Telling the main thread about {line!r}")
@@ -296,7 +289,7 @@ async def start_listener(cfg: Config) -> None:
             )
 
             print(f"[{peer}] Writing something back...")
-            line = "There!\n".encode("UTF-8")
+            line = b"There!\n"
             writer.writelines([line])
             await writer.drain()
             print(f"[{peer}] Telling the main thread about {line!r}")
@@ -307,12 +300,12 @@ async def start_listener(cfg: Config) -> None:
             await writer.wait_closed()
             print(f"[{peer}] Telling the main thread done")
             await cfg.mainq.put(ClientDoneEvent(etype="listener", peer=peer))
-        except Exception as err:  # pylint: disable=broad-except
+        except Exception as err:  # noqa: BLE001
             print(f"[{peer}] Complaining about the client: {err}")
             await cfg.mainq.put(FatalEvent(etype="listener", reason=f"Handling {peer}: {err}"))
 
     try:
-        srv = await find_listening_port(6502, 6502 + 1000, client_connected)
+        srv: Final = await find_listening_port(6502, 6502 + 1000, client_connected)
         if not srv.sockets:
             raise Exception(f"[listener] Expected a listening socket, got {srv.sockets!r}")
         hostname, port = srv.sockets[0].getsockname()[:2]
@@ -321,7 +314,7 @@ async def start_listener(cfg: Config) -> None:
         print("[listener] Awaiting client connections...")
         await srv.serve_forever()
         print("[listener] Done.")
-    except Exception as err:  # pylint: disable=broad-except
+    except Exception as err:  # noqa: BLE001
         print(f"[listener] Complaining to the main thread: {err}")
         await cfg.mainq.put(FatalEvent(etype="listener", reason=f"Listener thread: {err}"))
 
@@ -340,7 +333,10 @@ async def cleanup_tasks(cfg: Config) -> None:
 async def cleanup_children(cfg: Config) -> None:
     """Kill any remaining child processes."""
     print(f"About to kill and wait for {len(cfg.children)} child process(es)")
-    waiters = [asyncio.create_task(proc.wait()) for proc in cfg.children.values()]
+    waiters: Final = [
+        asyncio.create_task(proc.wait(), name=f"wait for pid {proc.pid}")
+        for proc in cfg.children.values()
+    ]
 
     for pid, proc in cfg.children.items():
         print(f"- pid {pid}")
@@ -348,7 +344,7 @@ async def cleanup_children(cfg: Config) -> None:
             proc.kill()
         except ProcessLookupError:
             print("  - already finished, it seems")
-        except Exception as err:  # pylint: disable=broad-except
+        except Exception as err:  # noqa: BLE001
             print(f"  - {err!r}")
 
     print("Waiting for the processes to exit...")
@@ -356,54 +352,53 @@ async def cleanup_children(cfg: Config) -> None:
     print(f"Got processes' exit status: {wait_res!r}")
 
 
-# pylint: disable-next=invalid-name
-TEvent = TypeVar("TEvent", bound=Event)
+if typing.TYPE_CHECKING:
+    TEvent = TypeVar("TEvent", bound=Event)
 
 
 async def expect_event(
-    msgq: asyncio.Queue[Event],  # pylint: disable=unsubscriptable-object
+    msgq: asyncio.Queue[Event],
     tag: str,
-    evtype: Type[TEvent],
+    evtype: type[TEvent],
 ) -> TEvent:
     """Make sure the next event in the main queue is of that type."""
-    evt = await msgq.get()
+    evt: Final = await msgq.get()
     if not isinstance(evt, evtype):
         sys.exit(f"[{tag}] Expected {evtype.__name__}, got {evt!r}")
     return evt
 
 
-# pylint: disable-next=invalid-name
-TListenerClientEvent = TypeVar("TListenerClientEvent", bound=ListenerClientEvent)
+if typing.TYPE_CHECKING:
+    TListenerClientEvent = TypeVar("TListenerClientEvent", bound=ListenerClientEvent)
 
 
 async def expect_client_event(
-    msgq: asyncio.Queue[ListenerClientEvent],  # pylint: disable=unsubscriptable-object
+    msgq: asyncio.Queue[ListenerClientEvent],
     tag: str,
-    evtype: Type[TListenerClientEvent],
+    evtype: type[TListenerClientEvent],
 ) -> TListenerClientEvent:
     """Make sure the next event in the main queue is of that type."""
-    evt = await msgq.get()
+    evt: Final = await msgq.get()
     if not isinstance(evt, evtype):
         sys.exit(f"[{tag}] Expected {evtype.__name__}, got {evt!r}")
     return evt
 
 
-async def test_connect(cfg: Config, conn: TestConnection) -> None:
-    """Whee!"""
-    # pylint: disable=too-many-statements
+async def test_connect(cfg: Config, conn: TestConnection) -> None:  # noqa: PLR0915
+    """Connect to the stunnel listener, read/write data as requested."""
 
     async def wait() -> None:
         """Wait for a little while."""
-        await asyncio.sleep(random.randint(3, 10) / 10)
+        await asyncio.sleep(random.randint(3, 10) / 10)  # noqa: S311  # no cryptography here
 
-    tag = f"test_connect [{conn.hostname}]:{conn.port} {conn.idx}"
+    tag: Final = f"test_connect [{conn.hostname}]:{conn.port} {conn.idx}"
     try:
         print(f"[{tag}] Trying port {conn.port} encrypted {conn.encrypted}")
         await wait()
         try:
             if conn.encrypted:
                 print(f"[{tag}] Creating an SSL context")
-                ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+                ctx: Final = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
                 print(f"[{tag}] - cert required")
                 ctx.verify_mode = ssl.CERT_REQUIRED
                 print(f"[{tag}] - load_verify_locations()")
@@ -422,35 +417,37 @@ async def test_connect(cfg: Config, conn: TestConnection) -> None:
                     conn.port,
                     ssl=False,
                 )
-        except IOError as err:
+        except OSError as err:
             sys.exit(f"Failed to connect to {conn.hostname}:{conn.port}: {err}")
 
         sock_addr, sock_port = writer.get_extra_info("sockname")[:2]
-        local = f"[{sock_addr}]:{sock_port}"
+        local: Final = f"[{sock_addr}]:{sock_port}"
         print(f"[{tag}] Connected to the server: {local}")
         await wait()
         print(f"[{tag}] Sending something...")
-        line = f"Hello {conn.idx}\n".encode("UTF-8")
+        line = f"Hello {conn.idx}\n".encode()
         writer.writelines([line])
         await writer.drain()
 
         print(f"[{tag}] Waiting for the main thread to figure it out")
-        evt_conn = await expect_client_event(conn.msgq, "test_connect", ClientConnectedEvent)
+        evt_conn: Final = await expect_client_event(conn.msgq, "test_connect", ClientConnectedEvent)
         print(f"[{tag}] The listener acknowledged {evt_conn.peer}")
         if conn.encrypted:
             if local == evt_conn.peer:
                 sys.exit(f"[{tag}] expected something other than {local!r}")
         else:
-            if local != evt_conn.peer:
+            if local != evt_conn.peer:  # noqa: PLR5501  # clarity
                 sys.exit(f"[{tag}] expected {local!r}, got {evt_conn.peer!r}")
 
         print(f"[{tag}] Waiting for the server to receive it...")
-        evt_recv = await expect_client_event(conn.msgq, "test_connect", ClientReceivedDataEvent)
+        evt_recv: Final = await expect_client_event(
+            conn.msgq, "test_connect", ClientReceivedDataEvent
+        )
         if evt_recv.line != line:
             sys.exit(f"[{tag}] Send: expected {line!r}, got {evt_recv.line!r}")
 
         print(f"[{tag}] Waiting for the server to send something")
-        evt_sent = await expect_client_event(conn.msgq, "test_connect", ClientSentDataEvent)
+        evt_sent: Final = await expect_client_event(conn.msgq, "test_connect", ClientSentDataEvent)
         print(f"[{tag}] Trying to receive the actual data")
         line = await reader.readline()
         if line != evt_sent.line:
@@ -470,7 +467,7 @@ async def test_connect(cfg: Config, conn: TestConnection) -> None:
         print(f"[{tag}] Letting the main thread know we're done")
         await cfg.mainq.put(ConnectionDoneEvent(etype=tag, idx=conn.idx))
         print(f"[{tag}] Done")
-    except Exception as err:  # pylint: disable=broad-except
+    except Exception as err:  # noqa: BLE001
         print(f"[{tag}] Telling the main thread about {err}")
         await cfg.mainq.put(FatalEvent(etype=tag, reason="Something went wrong: {err}"))
 
@@ -479,7 +476,10 @@ async def find_stunnel_port(cfg: Config, hostname: str, port: int) -> int:
     """Find a suitable listening port for the stunnel server."""
 
     async def callback(reader: asyncio.StreamReader, writer: asyncio.StreamWriter) -> None:
-        """This should never be called... right?"""
+        """Make sure that the no-data-transfer "find a port" connection transfers no data.
+
+        This should never be called... right?
+        """
         print("[find_stunnel_port] the callback was invoked?!")
         await cfg.mainq.put(
             FatalEvent(
@@ -488,11 +488,11 @@ async def find_stunnel_port(cfg: Config, hostname: str, port: int) -> int:
             )
         )
 
-    srv = await find_listening_port(port + 1, port + 1001, callback, hostname=hostname)
+    srv: Final = await find_listening_port(port + 1, port + 1001, callback, hostname=hostname)
     if not srv.sockets:
         sys.exit(f"Expected a listening socket, got {srv.sockets!r}")
-    lport = srv.sockets[0].getsockname()[1]
-    assert isinstance(lport, int)
+    lport: Final = srv.sockets[0].getsockname()[1]
+    assert isinstance(lport, int)  # noqa: S101
     print(f"[find_listening_port] got {lport}, shutting down the listener")
     srv.close()
     return lport
@@ -502,7 +502,7 @@ async def prepare_config_file(
     cfg: Config, hostname: str, port: int, stunnel_port: int
 ) -> pathlib.Path:
     """Create a configuration file for stunnel."""
-    proc = await asyncio.create_subprocess_exec(
+    proc: Final = await asyncio.create_subprocess_exec(
         "install",
         "-m",
         "400",
@@ -510,11 +510,11 @@ async def prepare_config_file(
         str(cfg.certdir / "key.pem"),
         str(cfg.tempd / "key.pem"),
     )
-    rcode = await proc.wait()
+    rcode: Final = await proc.wait()
     if rcode != 0:
         sys.exit(f"Could not copy the key file, install exit code {rcode}")
 
-    contents = f"""
+    contents: Final = f"""
 pid = {cfg.tempd}/stunnel.pid
 foreground = yes
 
@@ -526,16 +526,13 @@ accept = {hostname}:{stunnel_port}
 connect = {hostname}:{port}
 """
 
-    cfgfile = cfg.tempd / "stunnel.conf"
+    cfgfile: Final = cfg.tempd / "stunnel.conf"
     cfgfile.write_text(contents, encoding="UTF-8")
     return cfgfile
 
 
-async def launch_stunnel(
-    cfg: Config, cfgfile: pathlib.Path
-) -> asyncio.subprocess.Process:  # pylint: disable=no-member
+async def launch_stunnel(cfg: Config, cfgfile: pathlib.Path) -> asyncio.subprocess.Process:
     """Launch the stunnel server with the specified config file."""
-
     return await asyncio.create_subprocess_exec(
         str(cfg.program),
         str(cfgfile),
@@ -554,68 +551,68 @@ async def stunnel_output(cfg: Config, p_out: asyncio.StreamReader) -> None:
             if not data:
                 raise Exception("The stunnel child process ended unexpectedly")
 
-            line = data.decode("UTF-8").rstrip("\r\n")
+            line = data.decode().rstrip("\r\n")
             print(f"[stunnel_output] Read {line!r}")
             if "Configuration successful" in line:
                 print("[stunnel_output] The main thread ought to know")
                 await cfg.mainq.put(ChildReadyEvent(etype="stunnel_output"))
-    except Exception as err:  # pylint: disable=broad-except
+    except Exception as err:  # noqa: BLE001
         print(f"[stunnel_output] Complaining to the main thread about {err}")
         await cfg.mainq.put(
             FatalEvent(etype="stunnel_output", reason=f"Something went wrong: {err}")
         )
 
 
-def validate_conns(conns: TestConnections) -> None:
+def validate_conns(conns: TestConnections) -> None:  # noqa: PLR0915
     """Ensure the connections state is consistent."""
 
     def validate_by_id_unresolved(idx: int, conn: TestConnection) -> None:
         """Validate a still-unresolved connection."""
-        weird_p = [
+        weird_p: Final = [
             (peer, pconn)
             for peer, pconn in conns.by_peer.items()
             if pconn is conn or pconn.idx == idx
         ]
         if weird_p:
-            sys.exit(f"by_id conn {conn!r} in by_peer " f"weird_p {weird_p!r}")
+            sys.exit(f"by_id conn {conn!r} in by_peer weird_p {weird_p!r}")
 
-        unres = conns.unresolved.get(idx)
+        unres: Final = conns.unresolved.get(idx)
         if unres is None:
             sys.exit(f"conn {conn!r} not in unresolved")
         if unres is not conn:
             sys.exit(f"by_id conn {conn!r} not the same as unresolved {unres!r}")
-        weird_i = [
+        weird_i: Final = [
             (uidx, uconn)
             for uidx, uconn in conns.unresolved.items()
             if (uconn is conn) != (uidx == idx)
         ]
         if weird_i:
-            sys.exit(f"by_id conn {conn!r} doubly unresolved weird_i {weird_i!r}")
+            sys.exit(f"by_id conn {conn!r} doubly unresolved {weird_i=!r}")
 
     def validate_by_id_resolved(idx: int, conn: TestConnection) -> None:
         """Validate an already-resolved connection."""
-        assert conn.peer is not None
-        res = conns.by_peer.get(conn.peer)
+        assert conn.peer is not None  # noqa: S101
+        res: Final = conns.by_peer.get(conn.peer)
         if res is None:
             sys.exit(f"by_id conn {conn!r} not in by_peer")
         if res is not conn:
             sys.exit(f"by_id conn {conn!r} not the same as by_peer {res!r}")
 
-        weird_p = [
+        weird_p: Final = [
             (pidx, pconn)
             for pidx, pconn in conns.by_peer.items()
             if (pidx == conn.peer) != (pconn is conn)
         ]
         if weird_p:
-            sys.exit(f"by_id conn {conn!r} doubly in by_peer weird_p {weird_p!r}")
+            sys.exit(f"by_id conn {conn!r} doubly in by_peer {weird_p=!r}")
 
-        weird_i = [
+        weird_i: Final = [
             (iidx, iconn)
             for iidx, iconn in conns.unresolved.items()
             if iidx == idx or iconn is conn
         ]
         if weird_i:
-            sys.exit(f"by_id conn {conn!r} in unresolved weird_i {weird_i!r}")
+            sys.exit(f"by_id conn {conn!r} in unresolved {weird_i=!r}")
 
         if conn.peer in conns.pending:
             sys.exit(f"by_id conn {conn!r} in pending {conns.pending[conn.peer]!r}")
@@ -623,12 +620,12 @@ def validate_conns(conns: TestConnections) -> None:
     def validate_by_id(idx: int, conn: TestConnection) -> None:
         """Validate a connection in the by_id mapping."""
         if conn.idx != idx:
-            sys.exit(f"by_id conn {conn!r} should have idx {idx!r}")
-        weird_i = [
+            sys.exit(f"by_id conn {conn!r} should have idx {idx=!r}")
+        weird_i: Final = [
             (iidx, iconn) for iidx, iconn in conns.by_id.items() if (iidx == idx) != (iconn is conn)
         ]
         if weird_i:
-            sys.exit(f"by_id conn {conn!r} double weird_i {weird_i!r}")
+            sys.exit(f"by_id conn {conn!r} double {weird_i=!r}")
 
         if conn.peer is None:
             validate_by_id_unresolved(idx, conn)
@@ -640,13 +637,13 @@ def validate_conns(conns: TestConnections) -> None:
 
     for peer, conn in sorted(conns.by_peer.items()):
         if conn.peer != peer:
-            sys.exit(f"by_peer conn {conn!r} should have peer {peer!r}")
+            sys.exit(f"by_peer conn {conn!r} should have {peer=!r}")
         if conn.idx not in conns.by_id:
             sys.exit(f"by_peer conn {conn!r} not in by_id")
 
     for idx, conn in conns.unresolved.items():
         if conn.idx != idx:
-            sys.exit(f"unresolved conn {conn!r} should have idx {idx!r}")
+            sys.exit(f"unresolved conn {conn!r} should have {idx=!r}")
         if idx not in conns.by_id:
             sys.exit(f"unresolved conn {conn!r} not in by_id")
 
@@ -658,10 +655,10 @@ def validate_conns(conns: TestConnections) -> None:
             sys.exit(f"pending peer {peer!r} weird events {others!r}")
 
         if peer in conns.by_peer:
-            sys.exit(f"pending peer {peer!r} events {events!r} in by_peer {conns.by_peer[peer]!r}")
+            sys.exit(f"pending peer {peer!r} {events=!r} in by_peer {conns.by_peer[peer]!r}")
 
 
-def start_connections(
+def start_connections(  # noqa: PLR0913
     cfg: Config,
     conns: TestConnections,
     hostname: str,
@@ -683,7 +680,9 @@ def start_connections(
         conns.by_id[idx] = conn
         conns.unresolved[idx] = conn
         validate_conns(conns)
-        cfg.tasks[f"{prefix}{idx}"] = asyncio.create_task(test_connect(cfg, conn))
+        cfg.tasks[f"{prefix}{idx}"] = asyncio.create_task(
+            test_connect(cfg, conn), name=f"test {prefix}{idx}"
+        )
 
 
 async def process_connections(cfg: Config, conns: TestConnections, *, prefix: str) -> None:
@@ -691,7 +690,7 @@ async def process_connections(cfg: Config, conns: TestConnections, *, prefix: st
 
     async def process_listener_event(evt: ListenerClientEvent) -> None:
         """Shuffle things around the conns structure."""
-        peer = evt.peer
+        peer: Final = evt.peer
         if peer in conns.by_peer:
             await conns.by_peer[peer].msgq.put(evt)
             return
@@ -699,11 +698,11 @@ async def process_connections(cfg: Config, conns: TestConnections, *, prefix: st
         if peer in conns.pending:
             conns.pending[peer].append(evt)
             if isinstance(evt, ClientReceivedDataEvent):
-                conn = conns.by_id.get(evt.idx)
+                conn: Final = conns.by_id.get(evt.idx)
                 if conn is None:
                     sys.exit(f"Listener reported unknown connection {evt!r}")
                 if conn.peer is not None:
-                    sys.exit(f"Listener reported bad conn {conn!r} {evt!r}")
+                    sys.exit(f"Listener reported bad conn {conn=!r} {evt=!r}")
                 conn.peer = peer
                 conns.by_peer[peer] = conns.unresolved.pop(evt.idx)
 
@@ -712,23 +711,23 @@ async def process_connections(cfg: Config, conns: TestConnections, *, prefix: st
             return
 
         if not isinstance(evt, ClientConnectedEvent):
-            sys.exit(f"Expected 'client connected' first, " f"got {evt!r}")
+            sys.exit(f"Expected 'client connected' first, got {evt!r}")
         conns.pending[peer] = [evt]
 
     async def process_connection_done(evt: ConnectionDoneEvent) -> None:
         """Remove a connection from the structure."""
-        conn = conns.by_id.get(evt.idx)
+        conn: Final = conns.by_id.get(evt.idx)
         if conn is None:
             sys.exit(f"No connection for {evt!r}")
         if conn.peer is None:
-            sys.exit(f"Connection done too early: evt {evt!r} conn {conn!r}")
+            sys.exit(f"Connection done too early: {evt=!r} {conn=!r}")
 
         del conns.by_id[evt.idx]
         del conns.by_peer[conn.peer]
 
-        task_name = f"{prefix}{evt.idx}"
+        task_name: Final = f"{prefix}{evt.idx}"
         print(f"[process_connections] Fetching task {task_name}")
-        task = cfg.tasks.pop(task_name)
+        task: Final = cfg.tasks.pop(task_name)
         print(f"[process_connections] Waiting for task {task_name}")
         await asyncio.gather(task)
         print(f"[process_connections] Done with task {task_name}")
@@ -750,22 +749,22 @@ async def process_connections(cfg: Config, conns: TestConnections, *, prefix: st
 
 
 async def main() -> None:
-    """Main program: parse arguments, prepare an environment, run tests."""
+    """Parse arguments, prepare an environment, run tests."""
     with parse_args() as cfg:
-        stunnel_version = await get_stunnel_version(cfg)
+        stunnel_version: Final = await get_stunnel_version(cfg)
         print(f"Got stunnel version {stunnel_version}")
 
         try:
             print("[main] Starting to do things")
-            cfg.tasks["listen"] = asyncio.create_task(start_listener(cfg))
+            cfg.tasks["listen"] = asyncio.create_task(start_listener(cfg), name="listener")
 
             print("[main] Awaiting the 'listener started' event")
-            evt_listener = await expect_event(cfg.mainq, "main", ListenerStartedEvent)
-            hostname = evt_listener.hostname
-            port = evt_listener.port
+            evt_listener: Final = await expect_event(cfg.mainq, "main", ListenerStartedEvent)
+            hostname: Final = evt_listener.hostname
+            port: Final = evt_listener.port
             print(f"[main] Apparently we are listening on [{hostname}]:{port}")
 
-            conns = TestConnections(by_id={}, by_peer={}, pending={}, unresolved={})
+            conns: Final = TestConnections(by_id={}, by_peer={}, pending={}, unresolved={})
 
             print("[main] Testing cleartext connections")
             validate_conns(conns)
@@ -773,24 +772,26 @@ async def main() -> None:
             await process_connections(cfg, conns, prefix="plain-")
 
             print("[main] Picking a listen address:port for stunnel")
-            stunnel_port = await find_stunnel_port(cfg, hostname, port)
+            stunnel_port: Final = await find_stunnel_port(cfg, hostname, port)
             print(f"[main] Will put stunnel at {hostname}:{stunnel_port}")
 
             print("[main] Preparing the stunnel config file")
-            cfgfile = await prepare_config_file(cfg, hostname, port, stunnel_port)
+            cfgfile: Final = await prepare_config_file(cfg, hostname, port, stunnel_port)
             print(f"[main] Using config file {cfgfile}")
             for line in cfgfile.read_text(encoding="UTF-8").splitlines():
                 print(f"[main] {line}")
             print("[main] End of the config file")
 
             print("[main] Launching the stunnel server")
-            proc = await launch_stunnel(cfg, cfgfile)
+            proc: Final = await launch_stunnel(cfg, cfgfile)
             print(f"[main] - got pid {proc.pid}")
             cfg.children[proc.pid] = proc
 
             print("[main] Launching the stunnel output pipe thread")
-            assert proc.stderr is not None
-            cfg.tasks["output"] = asyncio.create_task(stunnel_output(cfg, proc.stderr))
+            assert proc.stderr is not None  # noqa: S101
+            cfg.tasks["output"] = asyncio.create_task(
+                stunnel_output(cfg, proc.stderr), name="collect output"
+            )
             print("[main] Waiting for stunnel to start up")
             await expect_event(cfg.mainq, "main", ChildReadyEvent)
 
